local M = {}

local IDENTITY = vmath.matrix4()
local CLEAR_COLOR = vmath.vector4(0, 0, 0, 0)

M.verbose = false

M.last_size = { width = nil, height = nil }
M.render_targets = {}

function M.prepare_unitquad_rendering()
	render.set_view(IDENTITY)
	render.set_projection(IDENTITY)
end

function M.clear_render_target(name, clear_color)
	clear_color = clear_color or CLEAR_COLOR
	render.set_render_target(M.render_targets[name].render_target)
	render.clear({[render.BUFFER_COLOR_BIT] = clear_color, [render.BUFFER_DEPTH_BIT] = 1})
	render.set_render_target(render.RENDER_TARGET_DEFAULT)
end

function M.draw_to_render_target(name, predicate, material, constants)
	if material then
		render.enable_material(material)
	end
	render.set_render_target(M.render_targets[name].render_target)
	render.clear({[render.BUFFER_COLOR_BIT] = clear_color, [render.BUFFER_DEPTH_BIT] = 1})
	if constants then
		render.draw(predicate, constants)
	else
		render.draw(predicate)
	end
	render.set_render_target(render.RENDER_TARGET_DEFAULT)
	if material then
		render.disable_material(material)
	end
end

function M.draw_render_target_by_predicate(name, predicate, constants)
	constants = constants or render.constant_buffer()
	render.enable_texture(0, M.render_targets[name].render_target, render.BUFFER_COLOR_BIT)
	render.draw(predicate, constants) -- the predicate might be a unit quad
	render.disable_texture(0, M.render_targets[name].render_target, render.BUFFER_COLOR_BIT)
end

function M.check_window_size_change()
	local wndWidth = render.get_window_width()
	local wndHeight = render.get_window_height()
	if(wndWidth <= 0 or wndHeight <= 0) then
		return
	end
	if M.last_size.width ~= wndWidth or M.last_size.height ~= wndHeight then
		M.last_size.width = wndWidth
		M.last_size.height = wndHeight
		if M.verbose then pprint("RenderTarget: Window size changed! New size " .. M.last_size.width .. "x" .. M.last_size.height) end
		M.update_render_targets()
	end
end

function M.update_render_targets()
	if M.verbose then pprint("RenderTarget: Updating render target list!") end
	for k, v in pairs(M.render_targets) do
		M.render_targets[k].render_target = M.update_render_target(M.render_targets[k].render_target, M.render_targets[k].name, M.render_targets[k].size)
		if M.verbose then pprint("RenderTarget: " .. M.render_targets[k].name .. " updated!") end
	end
end

function M.create_render_target(name, size)
	if M.render_targets[name] ~= nil then
		pprint("RenderTarget: Warning - create_render_target - target name already exists: " .. name)
		return false
	end

	M.render_targets[name] = {}
	M.render_targets[name].name = name
	M.render_targets[name].size = size
	M.render_targets[name].render_target = M.update_render_target(nil, name, size)

	if M.verbose then pprint("RenderTarget: create_render_target - new render target created: " .. name) end
end

-- creates or updates a render target a size or the size of the window view
function M.update_render_target(render_target, name, size)
	if render_target ~= nil then
		render.delete_render_target(render_target)
	end

	if size == nil then
		size = {}
	end

	local target_width = size.x or render.get_window_width()
	local target_height = size.y or render.get_window_height()

	local color_params = { 
		format = render.FORMAT_RGBA,
		width = target_width,
		height = target_height,
		min_filter = render.FILTER_LINEAR,
		mag_filter = render.FILTER_LINEAR,
		u_wrap = render.WRAP_CLAMP_TO_EDGE,
		v_wrap = render.WRAP_CLAMP_TO_EDGE
	}

	local depth_params = { 
		format = render.FORMAT_DEPTH,
		width = target_width,
		height = target_height,
		u_wrap = render.WRAP_CLAMP_TO_EDGE,
		v_wrap = render.WRAP_CLAMP_TO_EDGE 
	}

	return render.render_target(name, {[render.BUFFER_COLOR_BIT] = color_params, [render.BUFFER_DEPTH_BIT] = depth_params})

end

function M.get_render_target(name)
	if M.render_targets[name] then
		return M.render_targets[name].render_target
	end
end

return M